<?php
/*
 * Plugin Name: Extra Settings for WooCommerce
 * Description: Provides numerous additional settings for WooCommerce store and Storefront theme.
 * Version: 1.0.1
 * Author: taisho
 * WC requires at least: 2.6.0
 * WC tested up to: 3.8.1
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) die;

register_activation_hook( __FILE__, 'eswc_activate' );
function eswc_activate() {  
	// Prevent plugin activation if the minimum PHP version requirement is not met.
	if ( version_compare( PHP_VERSION, '5.4', '<' ) ) {
		deactivate_plugins( basename( __FILE__ ) );
		$msg = '<p><strong>Extra Settings for WooCommerce</strong> requires PHP version 5.4 or greater. Your server runs ' . PHP_VERSION . '.</p>';
		wp_die( $msg, 'Plugin Activation Error',  array( 'response' => 200, 'back_link' => TRUE ) );
	}
	// Store time of first plugin activation (add_option does nothing if the option already exists).
	add_option( 'eswc_first_activate', time());
}

// Create settings class.
class eswc_settings {
	
	private $options;
	private $settings_page_name;
	private $settings_menu_name;

    public function __construct() {
		
		if ( is_admin() ) {
		
			$this->settings_page_name = 'woo-extra-settings';
			$this->settings_menu_name = 'Woo Extra Settings';
			
			// Initialize and register settings. 
			add_action( 'admin_init', [ $this, 'display_options' ] );
			// Add settings page.
			add_action( 'admin_menu', [ $this, 'add_settings_page' ] );		
			// Add settings link to plugins page.
			add_action( 'plugin_action_links_' . plugin_basename( __FILE__ ), [ $this, 'add_settings_link' ] );			
			// Scripts for settings page - color picker & dynamically show / hide based on checked options.
			add_action( 'admin_enqueue_scripts',  [ $this, 'admin_enqueue_scripts' ] );
			// Display a notice encouraging to rate the plugin if not dismissed.
			include_once( 'includes/eswc-feedback-notice.php' );
		
		}
		
		$this->settings_list = [
			// General
			'eswc_kill_gutenberg' 		=> [ 'val' => false, 'title' => __( 'Gutenberg', 'extra-settings-for-woocommerce' ), 			'type' => 'checkbox', 'tab' => 'general',
											 'descr' =>  __( 'Disable Gutenberg editor. Don\'t load Gutenberg-related stylesheets from WordPress core, WooCommerce and Storefront.', 'extra-settings-for-woocommerce' ) ],				
			'eswc_no_price_range' 		=> [ 'val' => false, 'title' => __( 'Price range', 'extra-settings-for-woocommerce' ), 			'type' => 'checkbox', 'tab' => 'general',
											 'descr' => __( 'Remove price range for variable and grouped products on all pages.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			// Account
			'eswc_redirect' 			=> [ 'val' => false, 'title' => __( 'Login redirect', 'extra-settings-for-woocommerce' ),		'type' => 'checkbox', 'tab' => 'account',
											 'descr' => __( 'After a successful login, instead of redirecting to "My account", redirect customers to previously visited page and users with admin access to dashboard.', 'extra-settings-for-woocommerce' ),
											 'info' => __( 'Customer roles: customer, subscriber. Admin access roles: administrator, shop_manager, editor, author, contributor.', 'extra-settings-for-woocommerce' ) ],
			'eswc_no_orders_item_count'	=> [ 'val' => false, 'title' => __( 'Order item count', 'extra-settings-for-woocommerce' ),		'type' => 'checkbox', 'tab' => 'account',
											 'descr' => __( 'Remove item quantity from orders in "Total" column.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			// Shop & Category
			'eswc_no_shop_title' 		=> [ 'val' => false, 'title' => __( 'Shop page title', 'extra-settings-for-woocommerce' ), 		'type' => 'checkbox', 'tab' => 'shop_category',
											 'descr' => __( 'Remove shop page title. "Title Remover" plugin doesn\'t work on shop page.', 'extra-settings-for-woocommerce' ) ],
			'eswc_no_product_sort' 		=> [ 'val' => false, 'title' => __( 'Product sort', 'extra-settings-for-woocommerce' ), 		'type' => 'checkbox', 'tab' => 'shop_category',
											 'descr' => __( 'Remove product sort dropdown on top left and bottom left of shop & category pages.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_no_result_count' 		=> [ 'val' => false, 'title' => __( 'Result count', 'extra-settings-for-woocommerce' ), 		'type' => 'checkbox', 'tab' => 'shop_category',
											 'descr' => __( 'Remove result count on top left and bottom left of shop & category pages.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_no_category_count' 	=> [ 'val' => false, 'title' => __( 'Category item count', 'extra-settings-for-woocommerce' ),	'type' => 'checkbox', 'tab' => 'shop_category',
											 'descr' => __( 'Remove count of products within a category.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_archive_columns' 		=> [ 'val' => 0, 	 'title' => __( 'Archive columns', 'extra-settings-for-woocommerce' ), 		'type' => 'number',   'tab' => 'shop_category',
											 'descr' => __( 'The number of columns on shop and category pages. 0 = don\'t change.', 'extra-settings-for-woocommerce' ) ],
			'eswc_products_per_page' 	=> [ 'val' => 0, 	 'title' => __( 'Products per page', 'extra-settings-for-woocommerce' ), 	'type' => 'number',   'tab' => 'shop_category',
											 'descr' => __( 'The maximum number of products displayed on one page. 0 = don\'t change.', 'extra-settings-for-woocommerce' ) ],
									
			// Product page
			'eswc_no_gallery_lightbox' 	=> [ 'val' => false, 'title' => __( 'Gallery lightbox', 'extra-settings-for-woocommerce' ), 	'type' => 'checkbox', 'tab' => 'product_page',
											 'descr' => __( 'Don\'t show lightbox when the main product image is clicked. Useful especially when an image zoom plugin is already used.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_gallery_columns' 		=> [ 'val' => 0, 	 'title' => __( 'Gallery columns', 'extra-settings-for-woocommerce' ), 		'type' => 'number',   'tab' => 'product_page',
											 'descr' => __( 'The number of columns in product thumbnail gallery. 0 = don\'t change.', 'extra-settings-for-woocommerce' ) ],
			'eswc_var_select_change' 	=> [ 'val' => false, 'title' => __( 'Variation select', 'extra-settings-for-woocommerce' ), 	'type' => 'checkbox', 'tab' => 'product_page',
											 'descr' => __( 'Change "Choose an option" text displayed before selecting a product variation.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_var_select_text' 		=> [ 'val' => __( 'Choose an option', 'extra-settings-for-woocommerce' ), 'title' => __( 'Variation select text', 'extra-settings-for-woocommerce' ), 'type' => 'text', 'tab' => 'product_page',											 
											 'arg' => ( get_option( 'eswc_settingz' )['eswc_var_select_change'] == 1 ) ? null : [ 'class' => 'hidden' ] ],
			'eswc_no_clear_selection'	=> [ 'val' => false, 'title' => __( 'Reset variations', 'extra-settings-for-woocommerce' ),		'type' => 'checkbox', 'tab' => 'product_page',
											 'descr' => __( 'Remove a button that sets product variations in dropdowns to default values.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],		
			'eswc_no_sku'				=> [ 'val' => false, 'title' => __( 'SKU', 'extra-settings-for-woocommerce' ), 					'type' => 'checkbox', 'tab' => 'product_page',
											 'descr' => __( 'Remove product SKU.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_no_category_link'		=> [ 'val' => false, 'title' => __( 'Category link', 'extra-settings-for-woocommerce' ), 		'type' => 'checkbox', 'tab' => 'product_page',
											 'descr' => __( 'Remove a link to product category page.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_no_related_products'	=> [ 'val' => false, 'title' => __( 'Related products', 'extra-settings-for-woocommerce' ), 	'type' => 'checkbox', 'tab' => 'product_page',
											 'descr' => __( 'Remove related products.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			// Cart								 
			'eswc_no_cart_images'  		=> [ 'val' => __( 'Keep the images', 'extra-settings-for-woocommerce' ),  'title' => __( 'Cart product images', 'extra-settings-for-woocommerce' ),   'type' => 'select', 'tab' => 'cart',
											 'descr' => __( 'Select where the product image thumbnails should be removed.', 'extra-settings-for-woocommerce' ),											 
											 'options' => [ __( 'Keep the images', 'extra-settings-for-woocommerce' ), __( 'Mini-cart', 'extra-settings-for-woocommerce' ), __( 'Mini-cart & Cart page', 'extra-settings-for-woocommerce' ) ] ],
			// Images
			'eswc_change_image_lib' 	=> [ 'val' => false, 'title' => __( 'Image editor library', 'extra-settings-for-woocommerce' ), 'type' => 'checkbox', 'tab' => 'images',
											 'descr' => __( 'Change an image manipulation library from GD to ImageMagick.', 'extra-settings-for-woocommerce' ),
											 'info' => __( 'Try this setting if some product images in admin are not displayed or you encounter errors on image upload.', 'extra-settings-for-woocommerce' ) ],			
			// Storefront
			'eswc_no_google_fonts' 		=> [ 'val' => false, 'title' => __( 'Google fonts', 'extra-settings-for-woocommerce' ), 		'type' => 'checkbox', 'tab' => 'storefront',
											 'descr' => __( 'Don\'t download Google fonts (Source Sans Pro).', 'extra-settings-for-woocommerce' ) ],
			'eswc_no_footer_credits' 	=> [ 'val' => false, 'title' => __( 'Footer credits', 'extra-settings-for-woocommerce' ), 		'type' => 'checkbox', 'tab' => 'storefront',
											 'descr' => __( 'Remove "Built with Storefront & WooCommerce" and a privacy policy link from the footer. Keep copyright symbol with site name and current year.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_no_mini_cart_qty' 	=> [ 'val' => false, 'title' => __( 'Mini-cart quantity', 'extra-settings-for-woocommerce' ), 	'type' => 'checkbox', 'tab' => 'storefront',
											 'descr' => __( 'Remove the quantity next to mini-cart icon in desktop view, leaving only the cart total.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],			
			'eswc_no_product_search' 	=> [ 'val' => false, 'title' => __( 'Product search box', 'extra-settings-for-woocommerce' ), 	'type' => 'checkbox', 'tab' => 'storefront',
											 'descr' => __( 'Remove product search box in desktop view.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
			'eswc_no_stock_smiley'		=> [ 'val' => false, 'title' => __( 'Stock status icon', 'extra-settings-for-woocommerce' ), 	'type' => 'checkbox', 'tab' => 'storefront',
											 'descr' => __( 'Remove a smiley face icon next to the number of products in stock.', 'extra-settings-for-woocommerce' ),
											 'tip' => true ],
		];
    }
	
	public function display_options() {
		
		$active_tab = isset( $_GET[ 'tab' ] ) ? sanitize_key( $_GET[ 'tab' ] ) : 'general';
		
		// Option group (section ID), option name (one row in database with an array for all settings), args (sanitized)
		register_setting( $active_tab . '_section', 'eswc_settingz', [ $this, 'sanitize' ] );
		// ID / title / callback / page
		add_settings_section( $active_tab . '_section', null, [ $this, $active_tab . '_section_cb' ], $this->settings_page_name );
		
		// Add setting fields from 'settings_list' array based on an active tab.
		$arr = $this->settings_list;
		foreach($arr as $key => $item) {
			if ( $active_tab == $arr[$key]['tab']  ) {
				
				$args = [ 'name' => $key ];
				if ( isset ( $arr[$key]['arg'] ) ) $args+= $arr[$key]['arg'];				
				
				if ( isset ( $arr[$key]['tip'] ) ) {
					$screenshot_url = trailingslashit(plugin_dir_url( __FILE__ )) . 'screenshots/' . $arr[$key]['title'] . '.png';
					$tip = '<span class="eswc_help_tip"><img src="' . esc_url($screenshot_url) . '"/></span>';
				} else {
					$tip = '';
				}
				
				add_settings_field (
					$key, // ID
					$arr[$key]['title'] . $tip, // Title and a help tip icon if exists
					[ $this, 'eswc_print_field' ], // Callback
					$this->settings_page_name, // Page
					$active_tab . '_section', // Section ID
					$args // Optional args
				);
			}
		}
	
	}

    public function add_settings_page() {
        // This page will be under "Settings"
        $this->plugin_hook_suffix = add_options_page(
            'Settings Admin', $this->settings_menu_name, 'manage_options', $this->settings_page_name, [ $this, 'create_settings_page' ]
        );
    }
	
	public function add_settings_link( $links ) {
		$links = array_merge( [
			'<a href="' . esc_url( admin_url( '/options-general.php?page=' . $this->settings_page_name ) ) . '">' . __( 'Settings' ) . '</a>'
		], $links );
		return $links;
	}
	
	public function admin_enqueue_scripts ( $page ) {
		if ( $page !== $this->plugin_hook_suffix )
			return;
		$plugin_slug = 'extra-settings-for-woocommerce';
		$plugin_short_slug = 'woo-extra-settings';
		wp_enqueue_style( 'wp-color-picker' );	
		wp_enqueue_script( $plugin_short_slug . '-admin', plugins_url() . '/' . $plugin_slug . '/js/' . $plugin_slug . '-admin' . '.js', [ 'wp-color-picker', 'jquery' ], '', true );	
	}
	
	/**
	 * Get the option that is saved or the default.
	 *
	 * @param string $index. The option we want to get.
	 */
	public function eswc_get_settings( $index = false ) {
		
		$arr = $this->settings_list;
		foreach($arr as $key => $item) {		
			$defaults[$key] = $arr[$key]['val'];
		}
			
		$settings = get_option( 'eswc_settingz', $defaults );

		if ( $index && isset( $settings[ $index ] ) ) {
			return $settings[ $index ];
		}

		return $settings;
	}
		
    public function create_settings_page() {
		
		$this->options = $this->eswc_get_settings();		
	
        ?>
        <div class="wrap">
			<div class="eswc_admin_links">
				<a href="https://wordpress.org/support/plugin/extra-settings-for-woocommerce/" target="_blank"><?php esc_html_e( 'Support & suggestions', 'extra-settings-for-woocommerce' );?></a>
				|
				<a href="https://wordpress.org/support/plugin/extra-settings-for-woocommerce/reviews/?rate=5#new-post" target="_blank"><?php esc_html_e( 'Rate this plugin', 'extra-settings-for-woocommerce' );?></a>				
			</div>
            <h1>Extra Settings For WooCommerce</h1>
			<?php
			$active_tab = isset( $_GET[ 'tab' ] ) ? sanitize_key( $_GET[ 'tab' ] ) : 'general';
			?>
			<h2 class="nav-tab-wrapper">
				<a href="?page=woo-extra-settings&tab=general" class="nav-tab <?php echo $active_tab == 'general' ? 'nav-tab-active' : ''; ?>"> <?php echo esc_html_e( 'General' , 'extra-settings-for-woocommerce' );?> </a>
				<a href="?page=woo-extra-settings&tab=account" class="nav-tab <?php echo $active_tab == 'account' ? 'nav-tab-active' : ''; ?>"> <?php echo esc_html_e( 'Account' , 'extra-settings-for-woocommerce' );?> </a>
				<a href="?page=woo-extra-settings&tab=shop_category" class="nav-tab <?php echo $active_tab == 'shop_category' ? 'nav-tab-active' : ''; ?>"> <?php echo esc_html_e( 'Shop & Category' , 'extra-settings-for-woocommerce' );?> </a>
				<a href="?page=woo-extra-settings&tab=product_page" class="nav-tab <?php echo $active_tab == 'product_page' ? 'nav-tab-active' : ''; ?>"> <?php echo esc_html_e( 'Product page' , 'extra-settings-for-woocommerce' );?> </a>
				<a href="?page=woo-extra-settings&tab=cart" class="nav-tab <?php echo $active_tab == 'cart' ? 'nav-tab-active' : ''; ?>"> <?php echo esc_html_e( 'Cart' , 'extra-settings-for-woocommerce' );?> </a>
				<a href="?page=woo-extra-settings&tab=images" class="nav-tab <?php echo $active_tab == 'images' ? 'nav-tab-active' : ''; ?>"> <?php echo esc_html_e( 'Images' , 'extra-settings-for-woocommerce' );?> </a>
				<a href="?page=woo-extra-settings&tab=storefront" class="nav-tab <?php echo $active_tab == 'storefront' ? 'nav-tab-active' : ''; ?>"> <?php echo esc_html_e( 'Storefront' , 'extra-settings-for-woocommerce' );?> </a>
			</h2>
			<form method="post" action="<?php echo esc_url( add_query_arg ( 'tab', $active_tab, admin_url( 'options.php' ) ) ); ?>">            
            <?php
                // Output nonce, action, and option_page fields for a settings page.			
				settings_fields( $active_tab . '_section' );				
				echo '<table class="form-table">';
				do_settings_fields ($this->settings_page_name, $active_tab . '_section');
				echo '</table>';			
				
                submit_button();
            ?>
            </form>
        </div>
        <?php
    }

    /**
     * Sanitize each setting field as needed
     *
     * @param array $input Contains all settings fields as array keys
     */

	public function sanitize( $input ) {
		
		$new_input = $this->eswc_get_settings();
		$arr = $this->settings_list;		
		$active_tab = isset( $_GET[ 'tab' ] ) ? sanitize_key( $_GET[ 'tab' ] ) : 'general';
		
		foreach($arr as $key => $item) {		
			if ( $active_tab == $arr[$key]['tab'] ) {
				switch ( $arr[$key]['type'] ) {
					case 'checkbox' :
						if( isset( $input[$key] ) ) {
							$new_input[$key] = ( $input[$key] == 1 ? 1 : 0 );
						} else {
							$new_input[$key] = 0;
						}
						break;
					case 'text' :
					case 'select' :
						if( isset( $input[$key] ) )
							$new_input[$key] = sanitize_text_field( $input[$key] );
						break;
					case 'number' :
						if( isset( $input[$key] ) )
							$new_input[$key] = absint( $input[$key] );
						break;					 
				}
			}
		}
	
        return $new_input;
    }

    /** 
     * Get the settings option array and print one of its values
     */
	 
	public function eswc_print_field(array $args) {
		 
		$field = $args['name'];
	
		switch ( $this->settings_list[$field]['type'] ) {
			
			case 'checkbox' :
			
				$fieldset = 
					'<fieldset>
						<label><input id="%1$s" type="checkbox" name="eswc_settingz[%1$s]" value="1" %2$s />%3$s</label>
					</fieldset>';					
				$fieldset .= isset ( $this->settings_list[$field]['info'] ) ? '<p class="description">%4$s</p>' : '';
				
				$info = isset ( $this->settings_list[$field]['info'] ) ? $this->settings_list[$field]['info'] : '';

				printf (
					$fieldset,
					esc_attr( $field ),
					isset( $this->options[$field] ) && ( 1 == $this->options[$field] )  ? 'checked="checked" ':'',
					$this->settings_list[$field]['descr'],
					$info
				);			
				break;
				
			case 'select' :
				
				$options = $this->settings_list[$field]['options'];
				foreach($options as $item) {			
					$items[ $item ] = $item;
				}			
				
				printf (					
					'<select id="%1$s" name="eswc_settingz[%1$s]">',
					esc_attr( $field )
				);
				
				foreach( $items as $value => $option ) {         
					printf (
						'<option value="%1$s" %2$s>%3$s</option>',
						esc_attr( $value ),
						selected( $value, $this->options[$field], false ),
						esc_html( $option )
					);
				}
				
				printf (
					'</select>
					<p class="description">%1$s</p>',
					$this->settings_list[$field]['descr']
				);
				break;
			
			case 'text' :
			case 'number' :
			
				$fieldset = '<input type="text" id="%1$s" name="eswc_settingz[%1$s]" value="%2$s" />';
				$fieldset .= isset ( $this->settings_list[$field]['descr'] ) ? '<p class="description">%3$s</p>' : '';				
				$descr = isset ( $this->settings_list[$field]['descr'] ) ? $this->settings_list[$field]['descr'] : '';
			
				printf (
					$fieldset,
					esc_attr( $field ),
					isset( $this->options[$field] ) ? esc_attr( $this->options[$field]) : '',
					$descr
				);			
				break;
		}	
	
	}
}

$eswc_settingz_page = new eswc_settings();

if( is_admin() ) {	
	
	// Change plugin settings page CSS.
	add_action( 'admin_head-settings_page_woo-extra-settings', 'eswc_settings_style' );	
	function eswc_settings_style() {
		
		$my_style = '
		input[type=checkbox], input[type=radio] {
			margin: -4px 8px 0 0;
		}
		input, select {
			margin: 1px 1px 1px 0;
		}	
		.form-table th {
			padding: 10px 0 10px 0;
			width: 150px;
		}
		.form-table td {
			padding: 5px 10px;
		}
		.form-table td p {
			margin-bottom: 6px;
		}
		.wrap h1 {		
			padding: 9px 0;
		}
		h2 {		
			float: left;
		}
		.wrap h2.nav-tab-wrapper {
			margin-bottom: 0.7em;
		}
		.nav-tab:first-child {
			margin-left: 0;
		}
		.eswc_admin_links {
			float: right;
			margin: 15px 50px 15px 0;
			vertical-align: middle;
		}
		.eswc_numeric input {
			float: left;
			width: 35px;
			text-align: center;	
		}
		.eswc_numeric p.description {
			float: left;
			margin-left: 10px;
		}
		.eswc_dropdown p.description {
			float: left;
			margin-left: 20px;
		}		
		.eswc_dropdown select {
			float: left;
		}
		.eswc_help_tip::after {
			font-family: dashicons;
			content:  "\f223";
			float: right;
			vertical-align: middle;
			font-weight: normal;
		}
		.eswc_help_tip img {			
			display: none;
			position: absolute;			
			left: 162px;
			margin-top: 5px;
			box-sizing: border-box;	
			border: 2px #de930a solid;					
		}
		.eswc_help_tip:hover :nth-child(1){
			display: block;		
		}
		';
		
		echo '<style>' . eswc_minify($my_style) . '</style>';
		
	}

}

// Only if WooCommerce is active (doesn't work for Github installations, which have version in folder name).
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) || ( get_site_option('active_sitewide_plugins') && array_key_exists( 'woocommerce/woocommerce.php', get_site_option('active_sitewide_plugins') ) ) ) {
	
	// Remove plugin prefix and dash "_" from argument keys.
	function eswc_replaceArrayKeys( $array ) {
		$replacedKeys = str_replace('eswc_', null, array_keys( $array ));
		return array_combine( $replacedKeys, $array );
	}	
	$args = eswc_replaceArrayKeys ( $eswc_settingz_page->eswc_get_settings() );
	
	// add_action( 'wp_enqueue_scripts', function() use ( $args ) { eswc_front_scripts( $args ); } );
	add_action( 'wp_head', function() use ( $args ) { eswc_apply_styles( $args ); } );
	
	// GENERAL
	
	if ( $args['kill_gutenberg'] == true ) {
		// Disable Gutenberg editor.
		add_filter( 'use_block_editor_for_post_type', '__return_false', 999 );
		// Don't load Gutenberg-related stylesheets.
		add_action( 'wp_enqueue_scripts', 'eswc_no_block_css', 999 );
		function eswc_no_block_css() {
			wp_dequeue_style( 'wp-block-library' ); // Wordpress core
			wp_dequeue_style( 'wp-block-library-theme' ); // Wordpress core
			wp_dequeue_style( 'wc-block-style' ); // WooCommerce
			wp_dequeue_style( 'storefront-gutenberg-blocks' ); // Storefront theme
		}
	}
	
	if ( $args['no_price_range'] == true ) {
		// Don't show price range for variable and grouped products.
		add_filter( 'woocommerce_variable_price_html', '__return_false', 999 );	
		add_filter( 'woocommerce_variable_sale_price_html', '__return_false', 999 );
		add_filter( 'woocommerce_grouped_price_html', '__return_false', 999 );
	}
		
	// ACCOUNT
	
	// Login redirect if setting is checked.			
	if ( $args['redirect'] == true) {
		
		// Use database instead of PHP sessions to avoid server caching problems.
		// Add a unique user ID as an URL query string to WooCommerce login page.
		// wp_get_referer() gets this ID on login redirect and obtains corresponding previous URL from database.
		// Store previous URL in database when
		add_action( 'template_redirect', 'eswc_append_user_id_query', 999 );		
		function eswc_append_user_id_query() {
			if( ! is_user_logged_in() && ! is_wc_endpoint_url( 'lost-password' ) && is_account_page() ) {
			
				if ( strpos ( esc_url( $_SERVER["REQUEST_URI"] ), '?eswc_id=' ) !== false) return;		
				$url_list = get_option( 'eswc_url_before_login', null );
				
				for ($eswc_id = 'aaaaa'; $eswc_id <= 'zzzzz'; $eswc_id++) {		
					if ( ! isset( $url_list[$eswc_id] ) ) {						
						if ( ! isset( $url_list ) ) {
							$url_list = [ $eswc_id => [ 'url' => wp_get_referer(), 'time' => time() ] ];																						
						} else {							
							$url_list += [ $eswc_id => [ 'url' => wp_get_referer(), 'time' => time() ] ];							
						}
						break;
					}
				}

				update_option( 'eswc_url_before_login', $url_list );		
				wp_redirect( esc_url_raw (get_permalink( get_option('woocommerce_myaccount_page_id') ) .'?eswc_id=' . $eswc_id ) );					
				exit;				
			}
		}
		
		// Redirect user after successful login.
		add_filter( 'woocommerce_login_redirect', 'eswc_user_redirect', 999, 2 );
		function eswc_user_redirect( $redirect, $user ) {
			// Get the first of all the roles assigned to the user.
			$role = $user->roles[0];
			$dashboard = admin_url();
	
			$previous_urls = get_option( 'eswc_url_before_login', null );		
			if ( isset( $previous_urls ) ) {
				
				// Get previous URL from database.
				// wp_get_referer() and wp_get_original_referer() always return false, so $_SERVER['HTTP_REFERER'] is used.
				$query = parse_url( $_SERVER['HTTP_REFERER'], PHP_URL_QUERY );
				parse_str( $query, $params );
	
				$previous_url = $previous_urls[$params['eswc_id']]['url'];
				
				// Delete database entry for current login.
				// Delete database entries created over 1 hour ago. They remain because login page visit didn't result in a successful login.
				$now = time();
				foreach ($previous_urls as $eswc_id => $val) {
					if ( $eswc_id == $params['eswc_id'] || (( $now - $val['time'] ) / 3600 > 1 ) ) {							
						unset( $previous_urls[$eswc_id] );					
					}				
				}
				update_option( 'eswc_url_before_login', $previous_urls );	
			}			
			
			if (in_array($role, [ 'administrator', 'shop_manager', 'editor', 'author', 'contributor' ])) {
				$redirect = $dashboard;
			// Redirect to previously visited page or home if unavailable.
			} elseif (in_array($role, [ 'customer', 'subscriber' ] )) {		
				$redirect = isset( $previous_url ) ? $previous_url : esc_url( home_url('/') );
			} else {			
				$redirect = isset( $previous_url ) ? $previous_url : esc_url( home_url('/') );			
			}	
			return $redirect;
		
		}
	}
	
	if ( $args['no_orders_item_count'] == true ) {	
		// Remove My account > Orders item count in table (gettext for singular, ngettext for plural).
		add_filter('ngettext', 'eswc_no_orders_item_count', 999, 3 );
		function eswc_no_orders_item_count( $translated, $text, $domain ) {
			switch ( $text ) {
				case '%1$s for %2$s item' :
					$translated = '%1$s';
					break;
				case '%1$s for %2$s items' :
					$translated = '%1$s';
					break;
			}
			return $translated;
		}
	}
	
	// SHOP & CATEGORY
	
	if ( $args['no_shop_title'] == true ) {
		// Remove title on shop page. Title Remover plugin doesn't work on shop page.
		add_filter( 'woocommerce_show_page_title', 'eswc_not_a_shop_page', 999 );
		function eswc_not_a_shop_page() {
			return boolval( ! is_shop() );
		}
	}
	
	if ( $args['no_product_sort'] == true ) {
		// Remove sorting dropdown on archive pages.
		add_action( 'init', 'eswc_no_product_sort', 999 );
		function eswc_no_product_sort() {
			remove_action( 'woocommerce_after_shop_loop', 'woocommerce_catalog_ordering' );
			remove_action( 'woocommerce_before_shop_loop', 'woocommerce_catalog_ordering' );
		}
	}
	
	if ( $args['no_result_count'] == true ) {
		// Remove "Showing X results" on shop and category pages.
		add_action( 'init', 'eswc_no_result_count', 999 );
		function eswc_no_result_count() {
			remove_action( 'woocommerce_after_shop_loop', 'woocommerce_result_count', 20 );
			remove_action( 'woocommerce_before_shop_loop', 'woocommerce_result_count', 20 );
		}
	}	
	
	if ( $args['no_category_count'] == true ) {
		// Remove category results count.
		add_filter( 'woocommerce_subcategory_count_html', '__return_null', 999 );
	}
			
	
	if ( $args['archive_columns'] != 0 ) {
		// Change the number of columns on shop and category pages.		
		add_filter( 'loop_shop_columns', function( $filter_args ) use ( $args ) {
				return eswc_archive_columns( $filter_args, $args );
			}
		, 999 );

		function eswc_archive_columns( $filter_args, $args ) {	
			return $args['archive_columns'];
		}		
	}
	
	if ( $args['products_per_page'] != 0 ) {
		// Change the maximum number of products displayed on one page.		
		add_filter( 'loop_shop_per_page', function( $filter_args ) use ( $args ) {
				return eswc_products_per_page( $filter_args, $args );
			}
		, 999 );

		function eswc_products_per_page( $filter_args, $args ) {	
			return $args['products_per_page'];
		}		

	}	

	// PRODUCT PAGE
	
	if ( $args['no_gallery_lightbox'] == true ) {
		// Disable lightbox when clicking on gallery images.
		add_action( 'after_setup_theme', 'eswc_no_gallery_lightbox', 999 );
		function eswc_no_gallery_lightbox() {
			remove_theme_support( 'wc-product-gallery-lightbox' );
		}
	}
	
	if ( $args['gallery_columns'] != 0 ) {
		// Change the number of columns in product thumbnail gallery.
		add_filter( 'woocommerce_product_thumbnails_columns', function( $filter_args ) use ( $args ) {
				return eswc_gallery_columns( $filter_args, $args );
			}
		, 999 );

		function eswc_gallery_columns( $filter_args, $args ) {	
			return $args['gallery_columns'];
		}		
	}
	
	if ( $args['var_select_change'] == true ) {
		// Change dropdown text when no product variation is selected. Default is "Choose an option".
		add_filter( 'woocommerce_dropdown_variation_attribute_options_args', function( $filter_args ) use ( $args ) {
				return eswc_var_dropdown_args( $filter_args, $args );
			}
		, 999 );

		function eswc_var_dropdown_args( $filter_args, $args ) {
			$filter_args['show_option_none'] = $args['var_select_text'];
			return $filter_args;
		}
	}
	
	// no_clear_selection - CSS	
	
	if ( $args['no_sku'] == true ) {
		// Remove SKU.
		add_filter( 'wc_product_sku_enabled', '__return_false', 999 );
	}
	
	if ( $args['no_category_link'] == true ) {
		// Don't show a link to product category page.
		add_action( 'init', 'eswc_no_category_link', 999 );
		function eswc_no_category_link() {
			remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_meta', 40 );
		}
	}
	
	if ( $args['no_related_products'] == true ) {
		// Don't show related products.
		// add_filter( 'woocommerce_product_related_posts_query', '__return_empty_array', 999 );
		add_action( 'init', 'eswc_no_related_products', 999 );
		function eswc_no_related_products() {
			remove_action( 'woocommerce_after_single_product_summary', 'woocommerce_output_related_products', 20 );
		}		
	}
	
	// CART
	
	if ( $args['no_cart_images'] != 'Keep the images' ) {
		// Remove the product thumbnail images from both Mini-cart and cart page or only mini-cart depending on selected option.
		add_action( 'init', function() use ( $args ) { eswc_no_cart_images( $args ); }, 999 );	
		function eswc_no_cart_images( $args ) {
			// If "Mini-cart" option is selected, don't apply the filter on cart and checkout pages.
			// If "Mini-cart & Cart page" option is selected, apply the filter on all pages.
			if ( $args['no_cart_images'] == 'Mini-cart' && ! is_cart() && ! is_checkout() ) return;			
			add_filter( 'woocommerce_cart_item_thumbnail', '__return_empty_string' );			
		}
	}
	
	// IMAGES
	
	if ( $args['change_image_lib'] == true ) {
		// Try this setting if some product images in admin are not displayed or you encounter errors on image upload. Changes an image manipulation library from GD to ImageMagick.
		add_filter( 'wp_image_editors', 'eswc_change_graphic_lib', 999 );
		function eswc_change_graphic_lib($array) {
			return [ 'WP_Image_Editor_GD', 'WP_Image_Editor_Imagick' ];
		}
	}
		
	// STOREFRONT
	
	if ( $args['no_google_fonts'] == true ) {
		// Don't download Google fonts (Source Sans Pro).
		add_action( 'wp_enqueue_scripts', 'eswc_no_google_fonts', 999 );
		function eswc_no_google_fonts() {
			wp_dequeue_style( 'storefront-fonts' );
		}
	}
	
	if ( $args['no_footer_credits'] == true ) {
	
		// Remove or replace "Built with Storefront & WooCommerce" and a privacy policy link in the footer.
		add_action( 'init', 'eswc_no_footer_credits', 999 );
		function eswc_no_footer_credits () {
			remove_action( 'storefront_footer', 'storefront_credit', 20 );
			add_action( 'storefront_footer', 'eswc_storefront_credit', 20 );
		}
		function eswc_storefront_credit() {			
			$content = '&copy; ' . get_bloginfo('name') . ' ' .	date( 'Y' );		
			?>
			<div class="site-info">
				<?php echo esc_html( apply_filters( 'storefront_copyright_text', $content ) ); ?>
			</div>
			<?php
		}
	}
	
	if ( $args['no_mini_cart_qty'] == true ) {
		// Remove the quantity next to Storefront mini cart icon in desktop view.
		// Pluggable function - if regular function is used, function defined error happens on activation due to plugin loading after theme.
		if ( ! function_exists( 'storefront_cart_link' ) ) {
			function storefront_cart_link() {
				?>
				<a class="cart-contents" href="<?php echo get_permalink(get_option('woocommerce_cart_page_id')); ?>">
				<?php echo wp_kses_data( WC()->cart->get_cart_subtotal() ); ?>
				</a>
				<?php
			}
		}
	}
	
	if ( $args['no_product_search'] == true ) {	
		// Remove Storefront search button.
		add_action( 'init', 'eswc_no_product_search', 999 );
		function eswc_no_product_search() {
			remove_action( 'storefront_header', 'storefront_product_search', 40 );		
		}
	}
	
	// no_stock_smiley - CSS
	
	function eswc_apply_styles( $args ) {
		
		// PRODUCT PAGE
		
		$my_style ='';
		
		if ( $args['no_clear_selection'] == true ) {
			// Remove a button that sets product variations in dropdowns to default values.
			$my_style .=
			'a.reset_variations {
				display: none!important;
			}';
		}
		
		if ( $args['no_stock_smiley'] == true ) {
			// Remove a smiley icon before stock value.
			if ( ! is_product() ) return;		
			$my_style .=
			'.woocommerce .product .stock::before {
				content: none;
			}';
		}	
		
		if ( $my_style != '' ) echo '<style>' . eswc_minify($my_style) . '</style>';
		
	}
	
/* 	function eswc_front_scripts( $args ) {	
	}
 */
}

function eswc_minify( $input ) { 
	$output = $input;
	// Remove whitespace
	$output = preg_replace('/\s*([{}|:;,])\s+/', '$1', $output);
	// Remove trailing whitespace at the start
	$output = preg_replace('/\s\s+(.*)/', '$1', $output);
	// Remove comments
	// $output = preg_replace('#/\*.*?\*/#s', '', $output);
	$output = preg_replace('/(?:(?:\/\*(?:[^*]|(?:\*+[^*\/]))*\*+\/)|(?:(?<!\:|\\\|\')\/\/.*))/', '', $output);
	return $output;
}